<?php

/**
 * Class JobsImporter - Import data to WP posts with custom fields
 *
 * @uses Pods framework
 * @package Biuro Jobs importer
 * @used-by \jobs-importer\job-importer.php
 */
class JobsImporter {

    private $data;

    // Biuro record unique key
    protected $primaryKey = 'livas-id';

    // Pods instance
    protected $pods;

    // Posts container in WP
    protected $podsType = 'job';

    // Additional post fields
    protected $postProperties = [
        'post_status'    => 'publish',
        'comment_status' => 'closed'
    ];


    public function __construct($data)
    {
        $this->data = $data;
        $this->setPods();
    }

    /**
     *  Set Pods instance
     */
    private function setPods()
    {
        $this->pods = pods($this->podsType);
    }

    /**
     *  Get related items mapping array and remove it from data array
     *
     * @param $data
     *
     * @return array
     */
    private function getRelations(&$data)
    {
        $relations = [];
        if (array_key_exists('related_items', $data)) {
            $relations = $data['related_items'];
            unset($data['related_items']);
        }

        return $relations;
    }

    /**
     *  Import posts from provided data: if post with given Id exists, it rewrites
     */
    public function import()
    {
        foreach ($this->data as $ad) {

            // Get related items
            $relations = $this->getRelations($ad);

            // Prepare data array fro importing
            $this->prepare($ad, $relations);

            // Unique key of post
            $primaryKey = $ad[$this->primaryKey];

            $pod = $this->getPod($primaryKey);

            if ($pod->total_found() > 0) {

                try {

                    $podId = $pod->save(array_merge($ad, $this->postProperties));
                    print_r("Old record ($this->idKey: $primaryKey) has been updated. <br>");

                } catch (Exception $e) {

                    print_r("Error while updating record ($this->idKey: $primaryKey). Error: $e->getMessage()<br>");
                }
            } else {

                try {

                    $podId = $this->pods->add(array_merge($ad, $this->postProperties));
                    print_r("New record ($this->idKey: $primaryKey) has been created. <br>");
                } catch (Exception $e) {

                    print_r("Error while creating record ($this->idKey: $primaryKey). Error: $e->getMessage()<br>");
                }
            }

            if (isset($podId)) {

                // Set post language
                $this->setPostLang($podId, $ad['lang']);

            }
        }
    }

    /**
     *  Creates related pods if needed and prepare data array fro import
     *
     * @param $ad
     * @param $relations
     */
    private function prepare(&$ad, $relations)
    {
        foreach ($ad as $key => $item) {

            $lang = $ad['lang'];

            if (is_array($item)) {

                $slug = array_keys($item)[0];
                $value = array_values($item)[0];

                $termId = $this->getTerm($key, $slug, $value, $lang);

                $ad[$key] = $slug;

                // Adding related item if exists
                if (array_key_exists($key, $relations)) {
                    $ad[$relations[$key]] = $termId;
                }
            }
        }
    }

    /**
     *  Find and return pod id, if not exist, create it
     *
     * @param $key
     * @param $slug
     * @param $value
     * @param $lang
     *
     * @return int
     */
    private function getTerm($key, $slug, $value, $lang)
    {
        $term = pods($key, $slug);

        if ( ! $term->exists()) {

            try {

                $termId = pods($key)->add([
                    'name' => $value,
                    'slug' => $slug,
                ]);

                if ($termId) {

                    $this->setTermLang($termId, $lang);

                    return $termId;
                }

            } catch (Exception $e) {

                print_r("Error while adding taxonomies ($key => $slug). Error: $e->getMessage()<br>");
            }
        }

        return $term->id();
    }

    /**
     *  Set language param for given pod, using Polylang library
     *
     * @param $termId
     * @param $lang
     */
    private function setPostLang($termId, $lang)
    {
        try {

            PLL()->model->post->set_language($termId, $lang);
            print_r("Set language code '$lang' for item. <br>");

        } catch (Exception $e) {

            print_r("Error while setting language code for item, error: $e->getMessage().<br>");
        }
    }

    /**
     *  Set language param for given taxonomy, using Polylang library
     *
     * @param $termId
     * @param $lang
     */
    private function setTermLang($termId, $lang)
    {
        try {

            // PLL()->model->post->set_language($termId, $lang);
            PLL()->model->term->save_translations($termId, $lang);
            print_r("Set language code '$lang' for item. <br>");

        } catch (Exception $e) {

            print_r("Error while setting language code for item, error: $e->getMessage().<br>");
        }
    }

    /**
     *  Get pod by given id
     *
     * @param $id
     *
     * @return mixed
     */
    private function getPod($id)
    {
        $params = [
            'where' =>  [
                'relation'  =>  'AND',
                [
                    'value' =>  (int) $id,
                    'key'   =>  $this->idKey,
                    'compare'   =>  '=',
                ]
            ]
        ];

        return $this->pods->find($params);
    }



}
